@@ -4,7 +4,7 @@ use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = "0.11";
+our $VERSION = "0.20";
 
 1;
 
@@ -28,7 +28,7 @@ Many code was rewritten and optimized by XS.
 
 Gazelle supports following features.
 
-- only supports HTTP/1.0. But does not support Keepalive.
+- supports HTTP/1.1. But does not have Keepalive.
 
 - ultra fast HTTP processing using picohttpparser
 
