@@ -13,7 +13,7 @@ use Server::Starter ();
 use Try::Tiny;
 use Guard;
 
-our $VERSION = "0.11";
+our $VERSION = "0.20";
 
 use XSLoader;
 XSLoader::load(__PACKAGE__, $VERSION);
@@ -58,7 +58,7 @@ sub new {
         max_workers          => $max_workers,
         min_reqs_per_child   => (
             defined $args{min_reqs_per_child}
-                ? $args{min_reqs_per_child} : undef,
+            ? $args{min_reqs_per_child} : undef,
         ),
         max_reqs_per_child   => (
             $args{max_reqs_per_child} || $args{max_requests} || 1000,
@@ -66,7 +66,7 @@ sub new {
         spawn_interval       => $args{spawn_interval} || 0,
         err_respawn_interval => (
             defined $args{err_respawn_interval}
-                ? $args{err_respawn_interval} : undef,
+            ? $args{err_respawn_interval} : undef,
         ),
     }, $class;
 
@@ -129,20 +129,20 @@ sub run {
                 exit 0
                     if ( $self->{can_exit} || $self->{term_received} > 1 );
             };
-            
             local $SIG{PIPE} = 'IGNORE';
         PROC_LOOP:
             while ( $proc_req_count < $max_reqs_per_child) {
                 $self->{can_exit} = 1;
                 if ( my ($conn, $buf, $env) = accept_psgi(
-                    fileno($self->{listen_sock}), $self->{timeout}, $self->{_listen_sock_is_tcp}, 
+                    fileno($self->{listen_sock}), $self->{timeout}, $self->{_listen_sock_is_tcp},
                     $self->{host} || 0, $self->{port} || 0
                 ) ) {
                     my $guard = guard { close_client($conn) };
                     $self->{can_exit} = 0;
                     ++$proc_req_count;
-                    my $res = $bad_response;
 
+                    my $res = $bad_response;
+                    my $chunked = do { no warnings; lc delete $env->{HTTP_TRANSFER_ENCODING} eq 'chunked' };
                     if (my $cl = $env->{CONTENT_LENGTH}) {
                         my $buffer = Stream::Buffered->new($cl);
                         while ($cl > 0) {
@@ -159,18 +159,50 @@ sub run {
                             $cl -= length $chunk;
                         }
                         $env->{'psgi.input'} = $buffer->rewind;
+                    } elsif ( $chunked ) {
+                        my $buffer = Stream::Buffered->new($cl);
+                        my $chunk_buffer = '';
+                        my $length;
+                    DECHUNK: while(1) {
+                            my $chunk = "";
+                            if ( length $buf ) {
+                                $chunk = $buf;
+                                $buf = '';
+                            }
+                            else {
+                                read_timeout(
+                                    $conn, \$chunk, 16384, 0, $self->{timeout})
+                                    or next PROC_LOOP;
+                            }
+
+                            $chunk_buffer .= $chunk;
+                            while ( $chunk_buffer =~ s/^(([0-9a-fA-F]+).*\015\012)// ) {
+                                my $trailer   = $1;
+                                my $chunk_len = hex $2;
+                                if ($chunk_len == 0) {
+                                    last DECHUNK;
+                                } elsif (length $chunk_buffer < $chunk_len + 2) {
+                                    $chunk_buffer = $trailer . $chunk_buffer;
+                                    last;
+                                }
+                                $buffer->print(substr $chunk_buffer, 0, $chunk_len, '');
+                                $chunk_buffer =~ s/^\015\012//;
+                                $length += $chunk_len;
+                            }
+                        }
+                        $env->{CONTENT_LENGTH} = $length;
+                        $env->{'psgi.input'} = $buffer->rewind;
                     } else {
                         $env->{'psgi.input'} = $null_io;
                     }
-                    
                     $res = Plack::Util::run_app $app, $env;
-                    
+                    my $use_chunked = $env->{"SERVER_PROTOCOL"} eq 'HTTP/1.1' ? 1 : 0;
                     if (ref $res eq 'ARRAY') {
-                        $self->_handle_response($res, $conn);
+                        $self->_handle_response($res, $conn, $use_chunked);
                     } elsif (ref $res eq 'CODE') {
                         $res->(sub {
-                                   $self->_handle_response($_[0], $conn);
-                               });
+                            $self->_handle_response($_[0], $conn, $use_chunked);
+                        });
                     } else {
                         die "Bad response $res";
                     }
@@ -184,7 +216,6 @@ sub run {
     $pm->wait_all_children;
 }
 
-
 sub _calc_minmax_per_child {
     my $self = shift;
     my ($max,$min) = @_;
@@ -196,35 +227,47 @@ sub _calc_minmax_per_child {
 }
 
 sub _handle_response {
-    my($self, $res, $conn) = @_;
+    my($self, $res, $conn, $use_chunked) = @_;
     my $status_code = $res->[0];
     my $headers = $res->[1];
     my $body = $res->[2];
-    
+
     if (defined $body && ref $body eq 'ARRAY' ) {
-        write_psgi_response($conn, $self->{timeout}, $status_code, $headers , $body);
+        write_psgi_response($conn, $self->{timeout}, $status_code, $headers , $body, $use_chunked);
         return;
     }
-    write_psgi_response($conn, $self->{timeout}, $status_code, $headers, []) or return;
+
+    write_psgi_response_header($conn, $self->{timeout}, $status_code, $headers, [], $use_chunked) or return;
 
     if (defined $body) {
         my $failed;
         Plack::Util::foreach(
             $body,
             sub {
-                unless ($failed) {
-                    write_all($conn, $_[0], 0, $self->{timeout})
-                        or $failed = 1;
+                return if $failed;
+                my $ret;
+                if ( $use_chunked ) {
+                    $ret = write_chunk($conn, $_[0], 0, $self->{timeout});
                 }
+                else {
+                    $ret = write_all($conn, $_[0], 0, $self->{timeout});
+                }
+                $failed = 1 if ! defined $ret;
             },
         );
+        write_all($conn, "0\015\012\015\012", 0, $self->{timeout}) if $use_chunked;
     } else {
         return Plack::Util::inline_object
             write => sub {
-                write_all($conn, $_[0], 0, $self->{timeout})
+                if ( $use_chunked ) {
+                    write_chunk($conn, $_[0], 0, $self->{timeout});
+                }
+                else {
+                    write_all($conn, $_[0], 0, $self->{timeout});
+                }
             },
             close => sub {
-                #none
+                write_all($conn, "0\015\012\015\012", 0, $self->{timeout}) if $use_chunked;
             };
     }
 }
